/*
* @(#)Execucao.java  1.0  30/03/2001
*	
* Aline Vieira de Mello
* Pontifcia Universidade Catlica do Rio Grande do Sul - Faculdade de Informtica
*/

import java.awt.event.*;
import java.awt.*;
import java.math.*;
import javax.swing.*;

/**
*<i>Execuo</i> executa as informaes contidas na lista Memria.
* Extende a classe Thread.
*/
public class Execucao extends Thread
{
	private Simulador sim;
	private int contlinha;
	private double tempo;
	private boolean halt;
	private Arquitetura arq;
	private String linha;
	private String codigo,codigo1,codigo2,codigo3,codigo4;
	private	String valorhex1,valorhex2,resultadohex;
	private	String valorbin1,valorbin2,resultadobin;
	private int valordec1,valordec2,resultadodec;
		
	public Execucao(Simulador sim,Arquitetura arq)
	{
		super();
		this.sim=sim;
		this.arq=arq;
		halt=false;
	}
	
	/**
	*Seta um valor boolean para Halt.
	*/
	public void setHalt(boolean b){halt=b;}
	
	/**
	*Captura o valor da memria que corresponde ao endereo armazenado
	*em PC.
	*/
	public void fetch()      
  { 
   	//Captura o PC e o converte para decimal.
   	valorhex1=capturaValor("PC");
		valordec1=sim.getConversao().hexa_decimal(valorhex1);
	
    //Seleciona a linha da tabela Memria que corresponde ao valor.
    sim.getJanela().getTabelaMemoria().setRowSelectionInterval(valordec1,valordec1);
    
    //Corre a barra conforme a posio da memria
    int valoranteriorbarra=sim.getJanela().getScrollPaneMemoria().getVerticalScrollBar().getValue();
    int valoratualbarra=(valordec1*17);
    if((valoratualbarra-valoranteriorbarra)>289) 
    	sim.getJanela().getScrollPaneMemoria().getVerticalScrollBar().setValue(valoratualbarra-289);
		if((valoratualbarra-valoranteriorbarra)<0)
			sim.getJanela().getScrollPaneMemoria().getVerticalScrollBar().setValue(valoranteriorbarra - valoratualbarra);

    
   	//Captura a instrucao correspondente ao endereco do PC.
   	NodoMemoria inst=sim.getMemoria().getMemoria(valorhex1);
  	if(inst!=null)
  	{
  		codigo=inst.getInstrucao();
			sim.getJanela().getReg("IR").setValReg(codigo);
			linha=inst.getLinha();
  	}
  	else
 		{
 			codigo="0000";
 			linha="";
 		}
		codigo1=(codigo.substring(0,1));
		codigo2=(codigo.substring(1,2));
		codigo3=(codigo.substring(2,3));
		codigo4=(codigo.substring(3,4));
		
		sim.getJanela().getReg("PC").incrementa();
  }

	/**
	*Executa as instrues contidas na memria,
	*/
	public void run()
	{
		NodoMemoria inst=sim.getMemoria().getInicio();	
		//Se for clicado ou pressionado o boto STEP executa uma instruo.
		if(sim.getJanela().getStep() && !halt)
		{
			fetch();
			executa();
		}
		//Se for clicado ou pressionado o boto RUN executa todas
		//instruo com um intervalo de tempo definido pela seleo do
		//CheckBox rpido, normal ou lento.
		if(sim.getJanela().getRun())
		{
			while(sim.getJanela().getRun() && !halt)
			{
				fetch(); 
				executa();
				tempo=sim.getJanela().getTempo();
				try
				{
					sleep((int)tempo*1000);
				}catch(InterruptedException e){}
				
			}
		}
	}

	/**
	*Seleciona e executa a instruo conforme o cdigo armazenado
	*na memria.
	*/
	public void executa()
	{
		String inst;
		String cod[];
		cod=new String[4];		
		
		cod[0]=codigo1;
		cod[1]=codigo2;
		cod[2]=codigo3;
		cod[3]=codigo4;

		inst=arq.selecionaInstrucao(cod);

		if(inst!=null)
		{
			//Incrementa o numero de clocks, conforme a instrucao executada
			int nclock= arq.getArqMont().procuraInstrucao(inst).getNclock();
			sim.getJanela().incrementaNclock(nclock);
			
			//incrementa o numero de instrucoes
			sim.getJanela().incrementaNinstrucao();

			if(inst.equalsIgnoreCase("ADD"))
				add();
			else if(inst.equalsIgnoreCase("SUB"))
				sub();
			else if(inst.equalsIgnoreCase("AND"))
				and();
			else if(inst.equalsIgnoreCase("OR"))
				or();
			else if(inst.equalsIgnoreCase("XOR"))
				xor();
			else if(inst.equalsIgnoreCase("ADDI"))
				addi();
			else if(inst.equalsIgnoreCase("SUBI"))
				subi();
			else if(inst.equalsIgnoreCase("LDL"))
				ldl();
			else if(inst.equalsIgnoreCase("LDH"))
				ldh();
			else if(inst.equalsIgnoreCase("SL"))
				sl();
			else if(inst.equalsIgnoreCase("SR"))
				sr();
			else if(inst.equalsIgnoreCase("NOT"))
				not();
			else if(inst.equalsIgnoreCase("POP"))
				pop();
			else if(inst.equalsIgnoreCase("LD"))
				ld();
			else if(inst.equalsIgnoreCase("PUSH"))
				push();
			else if(inst.equalsIgnoreCase("ST"))
				st();
			else if(inst.equalsIgnoreCase("LD2"))
				ld2();
			else if(inst.equalsIgnoreCase("ST2"))
				st2();
			else if(inst.equalsIgnoreCase("DIF"))
				dif();
			else if(inst.equalsIgnoreCase("EQL"))
				eql();
			else if(inst.equalsIgnoreCase("SUP"))
				sup();
			else if(inst.equalsIgnoreCase("INF"))
				inf();
			else if(inst.equalsIgnoreCase("LDSP"))
				ldsp();
			else if(inst.equalsIgnoreCase("NOP"))
				nop();
			else if(inst.equalsIgnoreCase("RTS"))
				rts();
			else if(inst.equalsIgnoreCase("HALT"))
				halt();
			else if(inst.equalsIgnoreCase("JMPR"))
				jmpr();
			else if(inst.equalsIgnoreCase("JSRR"))
				jsrr();
			else if(inst.equalsIgnoreCase("JMP"))
				jmp();
			else if(inst.equalsIgnoreCase("JSR"))
				jsr();
			else if(inst.equalsIgnoreCase("JMPD"))
				jmpd();
			else if(inst.equalsIgnoreCase("JSRD"))
				jsrd();
			else if(inst.equalsIgnoreCase("JMPRC"))
				jmprc();
			else if(inst.equalsIgnoreCase("JSRRC"))
				jsrrc();
			else if(inst.equalsIgnoreCase("JMPC"))
				jmpc();
			else if(inst.equalsIgnoreCase("JSRC"))
				jsrc();
			else if(inst.equalsIgnoreCase("JMPDC"))
				jmpdc();
			else if(inst.equalsIgnoreCase("JSRDC"))
				jsrdc();
			
			else if(inst.equalsIgnoreCase("JN"))
				jn();
			else if(inst.equalsIgnoreCase("JV"))
				jv();
			else if(inst.equalsIgnoreCase("JC"))
				jc();
			else if(inst.equalsIgnoreCase("JZ"))
				jz();
			else if(inst.equalsIgnoreCase("JNR"))
				jnr();
			else if(inst.equalsIgnoreCase("JVR"))
				jvr();
			else if(inst.equalsIgnoreCase("JCR"))
				jcr();
			else if(inst.equalsIgnoreCase("JZR"))
				jzr();
			else if(inst.equalsIgnoreCase("JND"))
				jnd();
			else if(inst.equalsIgnoreCase("JVD"))
				jvd();
			else if(inst.equalsIgnoreCase("JCD"))
				jcd();
			else if(inst.equalsIgnoreCase("JZD"))
				jzd();
			else if(inst.equalsIgnoreCase("SL0"))
				sl0();
			else if(inst.equalsIgnoreCase("SL1"))
				sl1();
			else if(inst.equalsIgnoreCase("SR0"))
				sr0();
			else if(inst.equalsIgnoreCase("SR1"))
				sr1();
			else if(inst.equalsIgnoreCase("INC"))
				inc();
			else if(inst.equalsIgnoreCase("DEC"))
				dec();
			else if(inst.equalsIgnoreCase("NEG"))
				neg();
			else if(inst.equalsIgnoreCase("MOV"))
				mov();
		}
		else
			JOptionPane.showMessageDialog(sim.getJanela(),"Instrucao Inexistente "+codigo1+codigo2+codigo3+codigo4,"ERRO", JOptionPane.ERROR_MESSAGE);
	}
	
		
	/**
	*Captura o valor do codigo3 e do codigo4, soma e armazena
	*o resultado no registrador representado pelo codigo2.
	*/
	public void add()
	{
		valorhex1=capturaValor(codigo3);
		valorhex2=capturaValor(codigo4);
		valordec1=sim.getConversao().hexa_decimal(valorhex1);
		valordec2=sim.getConversao().hexa_decimal(valorhex2);
		resultadodec=soma(valordec1,valordec2);
		resultadohex=sim.getConversao().decimal_hexa(resultadodec);
		if(resultadohex.length()>4)
			resultadohex=resultadohex.substring(1,5);
		sim.getJanela().getReg("R"+codigo2).setValReg(resultadohex);
		
		tratandoFlags("ADD",resultadohex,valorhex1,valorhex2);
	}
	
	/**
	*Captura o valor do codigo3 e do codigo4, subtrai e armazena
	*o resultado no registrador representado pelo codigo2.
	*/
	public void sub()
	{
		valorhex1=capturaValor(codigo3);
		valorhex2=capturaValor(codigo4);
		valordec1=sim.getConversao().hexa_decimal(valorhex1);
		valordec2=sim.getConversao().hexa_decimal(valorhex2);
		resultadodec=valordec1-valordec2;
		resultadohex=sim.getConversao().decimal_hexa(resultadodec);
		sim.getJanela().getReg("R"+codigo2).setValReg(resultadohex);
		
		tratandoFlags("SUB",resultadohex,valorhex1,valorhex2);
	}

	/**
	*Captura o valor do codigo3 e do codigo4, realiza a operao AND
	*e armazena o resultado no registrador representado pelo codigo2.
	*/
	public void and()
	{
		String zero="0";
		int tam;
		
		valorhex1=capturaValor(codigo3);
		valorhex2=capturaValor(codigo4);
		valordec1=sim.getConversao().hexa_decimal(valorhex1);
		valordec2=sim.getConversao().hexa_decimal(valorhex2);
		valorbin1=sim.getConversao().decimal_binario(valordec1);
		valorbin2=sim.getConversao().decimal_binario(valordec2);
		
		tam=16-valorbin1.length();
		for(int i=0;i<tam;i++)
			valorbin1=zero.concat(valorbin1);
		
		tam=16-valorbin2.length();
		for(int i=0;i<tam;i++)
			valorbin2=zero.concat(valorbin2);
		
		resultadobin=e(valorbin1,valorbin2);
		resultadodec=sim.getConversao().binario_decimal(resultadobin);
		resultadohex=sim.getConversao().decimal_hexa(resultadodec);
		sim.getJanela().getReg("R"+codigo2).setValReg(resultadohex);
	
  	tratandoFlags("AND",resultadohex,valorhex1,valorhex2);
	}

	/**
	*Captura o valor do codigo3 e do codigo4, realiza a operao OR
	*e armazena o resultado no registrador representado pelo codigo2.
	*/
	public void or()
	{
		String zero="0";
		int tam;
		
		valorhex1=capturaValor(codigo3);
		valorhex2=capturaValor(codigo4);
		valordec1=sim.getConversao().hexa_decimal(valorhex1);
		valordec2=sim.getConversao().hexa_decimal(valorhex2);
		valorbin1=sim.getConversao().decimal_binario(valordec1);
		valorbin2=sim.getConversao().decimal_binario(valordec2);
		
		tam=16-valorbin1.length();
		for(int i=0;i<tam;i++)
			valorbin1=zero.concat(valorbin1);
		
		tam=16-valorbin2.length();
		for(int i=0;i<tam;i++)
			valorbin2=zero.concat(valorbin2);
		
		resultadobin=or(valorbin1,valorbin2);
		resultadodec=sim.getConversao().binario_decimal(resultadobin);
		resultadohex=sim.getConversao().decimal_hexa(resultadodec);
		sim.getJanela().getReg("R"+codigo2).setValReg(resultadohex);
		
		tratandoFlags("OR",resultadohex,valorhex1,valorhex2);
	}

	/**
	*Captura o valor do codigo3 e do codigo4, realiza a operao XOR
	*e armazena o resultado no registrador representado pelo codigo2.
	*/
	public void xor()
	{
		String zero="0";
		int tam;
		
		valorhex1=capturaValor(codigo3);
		valorhex2=capturaValor(codigo4);
		valordec1=sim.getConversao().hexa_decimal(valorhex1);
		valordec2=sim.getConversao().hexa_decimal(valorhex2);
		valorbin1=sim.getConversao().decimal_binario(valordec1);
		valorbin2=sim.getConversao().decimal_binario(valordec2);
		
		tam=16-valorbin1.length();
		for(int i=0;i<tam;i++)
			valorbin1=zero.concat(valorbin1);
		
		tam=16-valorbin2.length();
		for(int i=0;i<tam;i++)
			valorbin2=zero.concat(valorbin2);
		
		resultadobin=xor(valorbin1,valorbin2);
		resultadodec=sim.getConversao().binario_decimal(resultadobin);
		resultadohex=sim.getConversao().decimal_hexa(resultadodec);
		sim.getJanela().getReg("R"+codigo2).setValReg(resultadohex);

		tratandoFlags("XOR",resultadohex,valorhex1,valorhex2);
	}

	/**
	*Captura o valor do codigo2 e soma a extenso do sinal do
	*codigo3 e armazena o resultado no registrador representado pelo
	*codigo2.
	*/
	public void addi()
	{
		valorhex1=capturaValor(codigo2);
		valorhex2=ext_sinal(codigo3);
		valordec1=sim.getConversao().hexa_decimal(valorhex1);
		valordec2=sim.getConversao().hexa_decimal(valorhex2);
		resultadodec=soma(valordec1,valordec2);
		resultadohex=sim.getConversao().decimal_hexa(resultadodec);
		if(resultadohex.length()>4)
			resultadohex=resultadohex.substring(1,5);
		sim.getJanela().getReg("R"+codigo2).setValReg(resultadohex);

  	tratandoFlags("ADDI",resultadohex,valorhex1,valorhex2);
	}

	/**
	*Captura o valor do codigo2 e subtrai da extenso do sinal do
	*codigo3 e armazena o resultado no registrador representado pelo
	*codigo2.
	*/
	public void subi()
	{
		valorhex1=capturaValor(codigo2);
		valorhex2=ext_sinal(codigo3);
		valordec1=sim.getConversao().hexa_decimal(valorhex1);
		valordec2=sim.getConversao().hexa_decimal(valorhex2);
		resultadodec=valordec1-valordec2;
		resultadohex=sim.getConversao().decimal_hexa(resultadodec);
		sim.getJanela().getReg("R"+codigo2).setValReg(resultadohex);
		
		tratandoFlags("SUBI",resultadohex,valorhex1,valorhex2);
	}

	/**
	*Captura o valor do codigo2 e realiza a operao AND com a
	*extenso do sinal do codigo3 e armazena o resultado no
	*registrador representado pelo codigo2.
	*/
	public void andi()
	{
		String zero="0";
		int tam;
		
		valorhex1=capturaValor(codigo2);
		valorhex2=ext_sinal(codigo3);
		valordec1=sim.getConversao().hexa_decimal(valorhex1);
		valordec2=sim.getConversao().hexa_decimal(valorhex2);
		valorbin1=sim.getConversao().decimal_binario(valordec1);
		valorbin2=sim.getConversao().decimal_binario(valordec2);
		
		tam=16-valorbin1.length();
		for(int i=0;i<tam;i++)
			valorbin1=zero.concat(valorbin1);
		
		tam=16-valorbin2.length();
		for(int i=0;i<tam;i++)
			valorbin2=zero.concat(valorbin2);
		
		resultadobin=e(valorbin1,valorbin2);
		resultadodec=sim.getConversao().binario_decimal(resultadobin);
		resultadohex=sim.getConversao().decimal_hexa(resultadodec);
		sim.getJanela().getReg("R"+codigo2).setValReg(resultadohex);
		
		tratandoFlags("ANDI",resultadohex,valorhex1,valorhex2);
	}

	/**
	*Captura o valor do codigo2 e realiza a operao OR com a
	*extenso do sinal do codigo3 e armazena o resultado no
	*registrador representado pelo codigo2.
	*/
	public void ori()
	{
		String zero="0";
		int tam;
		
		valorhex1=capturaValor(codigo2);
		valorhex2=ext_sinal(codigo3);
		valordec1=sim.getConversao().hexa_decimal(valorhex1);
		valordec2=sim.getConversao().hexa_decimal(valorhex2);
		valorbin1=sim.getConversao().decimal_binario(valordec1);
		valorbin2=sim.getConversao().decimal_binario(valordec2);
		
		tam=16-valorbin1.length();
		for(int i=0;i<tam;i++)
			valorbin1=zero.concat(valorbin1);
		
		tam=16-valorbin2.length();
		for(int i=0;i<tam;i++)
			valorbin2=zero.concat(valorbin2);
		
		resultadobin=or(valorbin1,valorbin2);
		resultadodec=sim.getConversao().binario_decimal(resultadobin);
		resultadohex=sim.getConversao().decimal_hexa(resultadodec);
		sim.getJanela().getReg("R"+codigo2).setValReg(resultadohex);
		
		tratandoFlags("ORI",resultadohex,valorhex1,valorhex2);
	}

	/**
	*Captura o valor do codigo2 e realiza a operao XOR com a
	*extenso do sinal do codigo3 e armazena o resultado no
	*registrador representado pelo codigo2.
	*/
	public void xori()
	{
		String zero="0";
		int tam;
		
		valorhex1=capturaValor(codigo2);
		valorhex2=ext_sinal(codigo3);
		valordec1=sim.getConversao().hexa_decimal(valorhex1);
		valordec2=sim.getConversao().hexa_decimal(valorhex2);
		valorbin1=sim.getConversao().decimal_binario(valordec1);
		valorbin2=sim.getConversao().decimal_binario(valordec2);
		
		tam=16-valorbin1.length();
		for(int i=0;i<tam;i++)
			valorbin1=zero.concat(valorbin1);
		
		tam=16-valorbin2.length();
		for(int i=0;i<tam;i++)
			valorbin2=zero.concat(valorbin2);
		
		resultadobin=xor(valorbin1,valorbin2);
		resultadodec=sim.getConversao().binario_decimal(resultadobin);
		resultadohex=sim.getConversao().decimal_hexa(resultadodec);
		sim.getJanela().getReg("R"+codigo2).setValReg(resultadohex);
		
		tratandoFlags("XORI",resultadohex,valorhex1,valorhex2);
	}

	/**
	*Captura o valor da parte alta do codigo2 e concatena com o valor
	*armazenado no codigo3 e codigo4.O resultado da concatenao 
	*armazenado no registrador representado pelo codigo2.
	*/
	public void ldh()
	{
		valorhex1=capturaValor(codigo2);
		valorhex2=codigo3.concat(codigo4);
		resultadohex=valorhex1.substring(2,4);
		resultadohex=valorhex2.concat(resultadohex);
		sim.getJanela().getReg("R"+codigo2).setValReg(resultadohex);
		
		tratandoFlags("LDH",resultadohex,valorhex1,valorhex2);
	}
	
	/**
	*Captura o valor da parte baixa do codigo2 e concatena com o valor
	*armazenado no codigo3 e codigo4.O resultado da concatenao 
	*armazenado no registrador representado pelo codigo2.
	*/
	public void ldl()
	{
		valorhex1=capturaValor(codigo2);
		valorhex2=codigo3.concat(codigo4);
		resultadohex=valorhex1.substring(0,2);
		resultadohex=resultadohex.concat(valorhex2);
		sim.getJanela().getReg("R"+codigo2).setValReg(resultadohex);
		
		tratandoFlags("LDL",resultadohex,valorhex1,valorhex2);
	}
	
	/**
	*Desloca para a esquerda o valor armazenado no codigo4 e concatena 
	*no bit menos significativo UM (caso o flag geral estiver setado
	*em true) ou ZERO (caso contrrio). 
	*/
	public void sl()
	{
		int tam;
		String zero="0";
		String flag_dep;
		flag_dep=arq.getArqMont().procuraInstrucao("SL").getFlag_dep();
		
		valorhex1=capturaValor(codigo4);
		valordec1=sim.getConversao().hexa_decimal(valorhex1);
		valorbin1=sim.getConversao().decimal_binario(valordec1);
		
		tam=16-valorbin1.length();
		for(int i=0;i<tam;i++)
			valorbin1=zero.concat(valorbin1);
		
		valorbin2="";
		for(int i=1;i<16;i++)
			valorbin2=valorbin2.concat(valorbin1.substring(i,i+1));
	
		if(sim.getJanela().getFlag(flag_dep).getValFlag())
			valorbin2=valorbin2.concat("1");
		else	
			valorbin2=valorbin2.concat("0");
	
  	resultadodec=sim.getConversao().binario_decimal(valorbin2);
		resultadohex=sim.getConversao().decimal_hexa(resultadodec);
		sim.getJanela().getReg("R"+codigo2).setValReg(resultadohex);
		
		tratandoFlags("SL",resultadohex,valorhex1,"0000");
	}

	/**
	*Desloca para a esquerda o valor armazenado no codigo4 e concatena 
	*no bit menos significativo ZERO.
	*/
	public void sl0()
	{
		int tam;
		String zero="0";
		
		valorhex1=capturaValor(codigo4);
		valordec1=sim.getConversao().hexa_decimal(valorhex1);
		valorbin1=sim.getConversao().decimal_binario(valordec1);
		
		tam=16-valorbin1.length();
		for(int i=0;i<tam;i++)
			valorbin1=zero.concat(valorbin1);
		
		valorbin2="";
		for(int i=1;i<16;i++)
			valorbin2=valorbin2.concat(valorbin1.substring(i,i+1));
	
		valorbin2=valorbin2.concat("0");
		resultadodec=sim.getConversao().binario_decimal(valorbin2);
		resultadohex=sim.getConversao().decimal_hexa(resultadodec);
		sim.getJanela().getReg("R"+codigo2).setValReg(resultadohex);
		
		tratandoFlags("SL0",resultadohex,valorhex1,"0000");
	}

	/**
	*Desloca para a esquerda o valor armazenado no codigo4 e concatena 
	*no bit menos significativo UM.
	*/
	public void sl1()
	{
		int tam;
		String zero="0";
		
		valorhex1=capturaValor(codigo4);
		valordec1=sim.getConversao().hexa_decimal(valorhex1);
		valorbin1=sim.getConversao().decimal_binario(valordec1);
		
		tam=16-valorbin1.length();
		for(int i=0;i<tam;i++)
			valorbin1=zero.concat(valorbin1);
		
		valorbin2="";
		for(int i=1;i<16;i++)
			valorbin2=valorbin2.concat(valorbin1.substring(i,i+1));
	
		valorbin2=valorbin2.concat("1");
		resultadodec=sim.getConversao().binario_decimal(valorbin2);
		resultadohex=sim.getConversao().decimal_hexa(resultadodec);
		sim.getJanela().getReg("R"+codigo2).setValReg(resultadohex);
		
		tratandoFlags("SL1",resultadohex,valorhex1,"0000");
	}

	/**
	*Desloca para a direita o valor armazenado no codigo4 e concatena 
	*no bit mais significativo UM (caso o flag geral estiver setado
	*em true) ou ZERO (caso contrrio). 
	*/
	public void sr()
	{
		int tam;
		String zero="0";
		String flag_dep;
		flag_dep=arq.getArqMont().procuraInstrucao("SL").getFlag_dep();
		
		valorhex1=capturaValor(codigo4);
		valordec1=sim.getConversao().hexa_decimal(valorhex1);
		valorbin1=sim.getConversao().decimal_binario(valordec1);
		
		tam=16-valorbin1.length();
		for(int i=0;i<tam;i++)
			valorbin1=zero.concat(valorbin1);
		
		valorbin2="";
		for(int i=0;i<15;i++)
			valorbin2=valorbin2.concat(valorbin1.substring(i,i+1));
	
		if(sim.getJanela().getFlag(flag_dep).getValFlag())
			valorbin2="1".concat(valorbin2);
		else	
			valorbin2="0".concat(valorbin2);
	
		resultadodec=sim.getConversao().binario_decimal(valorbin2);
		resultadohex=sim.getConversao().decimal_hexa(resultadodec);
		sim.getJanela().getReg("R"+codigo2).setValReg(resultadohex);
		
		tratandoFlags("SR",resultadohex,valorhex1,"0000");
	}

	/**
	*Desloca para a direita o valor armazenado no codigo4 e concatena 
	*no bit mais significativo ZERO. 
	*/
	public void sr0()
	{
		int tam;
		String zero="0";
		
		valorhex1=capturaValor(codigo4);
		valordec1=sim.getConversao().hexa_decimal(valorhex1);
		valorbin1=sim.getConversao().decimal_binario(valordec1);
		
		tam=16-valorbin1.length();
		for(int i=0;i<tam;i++)
			valorbin1=zero.concat(valorbin1);
		
		valorbin2="";
		for(int i=0;i<15;i++)
			valorbin2=valorbin2.concat(valorbin1.substring(i,i+1));
	
		valorbin2="0".concat(valorbin2);
	
		resultadodec=sim.getConversao().binario_decimal(valorbin2);
		resultadohex=sim.getConversao().decimal_hexa(resultadodec);
		sim.getJanela().getReg("R"+codigo2).setValReg(resultadohex);

		tratandoFlags("SR0",resultadohex,valorhex1,"0000");
	}

	/**
	*Desloca para a direita o valor armazenado no codigo4 e concatena 
	*no bit mais significativo UM. 
	*/
	public void sr1()
	{
		int tam;
		String zero="0";
		
		valorhex1=capturaValor(codigo4);
		valordec1=sim.getConversao().hexa_decimal(valorhex1);
		valorbin1=sim.getConversao().decimal_binario(valordec1);
		
		tam=16-valorbin1.length();
		for(int i=0;i<tam;i++)
			valorbin1=zero.concat(valorbin1);
		
		valorbin2="";
		for(int i=0;i<15;i++)
			valorbin2=valorbin2.concat(valorbin1.substring(i,i+1));
	
		valorbin2="1".concat(valorbin2);
	
		resultadodec=sim.getConversao().binario_decimal(valorbin2);
		resultadohex=sim.getConversao().decimal_hexa(resultadodec);
		sim.getJanela().getReg("R"+codigo2).setValReg(resultadohex);

		tratandoFlags("SR1",resultadohex,valorhex1,"0000");
	}
	
	/**
	*Captura o valor armazenado no registrador indicado pelo codigo4
	*e move para o registrador indicado pelo codigo2.
	*/
	public void mov()
	{
		valorhex1=capturaValor(codigo4);
		resultadodec=sim.getConversao().hexa_decimal(valorhex1);
		sim.getJanela().getReg("R"+codigo2).setValReg(valorhex1);

		tratandoFlags("MOV",valorhex1,valorhex1,"0000");
	}
	
	/**
	*Captura o valor armazenado no codigo4 e o subtrai do tamanho
	*da memoria -1. O resultado  armazenado no registrador representado
	*pelo codigo2.
	*/
	public void not()
	{
		valorhex1=capturaValor(codigo4);
		valordec1=sim.getConversao().hexa_decimal(valorhex1);
		resultadodec=sim.getTamanhoMemoria()-valordec1;
		resultadohex=sim.getConversao().decimal_hexa(resultadodec);
		sim.getJanela().getReg("R"+codigo2).setValReg(resultadohex);

		tratandoFlags("NOT",resultadohex,valorhex1,"0000");
	}

	/**
	*Captura o valor armazenado no codigo4 e o subtrai do tamanho
	*da memoria -1 para fazer o not e soma um para negar. O resultado  armazenado no registrador
	*representado pelo codigo2.
	*/
	public void neg()
	{
		valorhex1=capturaValor(codigo4);
		valordec1=sim.getConversao().hexa_decimal(valorhex1);
		resultadodec=sim.getTamanhoMemoria()-valordec1+1;
		resultadohex=sim.getConversao().decimal_hexa(resultadodec);
		sim.getJanela().getReg("R"+codigo2).setValReg(resultadohex);

		tratandoFlags("NOT",resultadohex,valorhex1,"0000");
	}

	/**
	*Captura o valor do codigo4 e incrementa armazenando
	*o resultado no registrador representado pelo codigo2.
	*/
	public void inc()
	{
		valorhex1=capturaValor(codigo4);
		valordec1=sim.getConversao().hexa_decimal(valorhex1);
		resultadodec=valordec1+1;
		resultadohex=sim.getConversao().decimal_hexa(resultadodec);
		sim.getJanela().getReg("R"+codigo2).setValReg(resultadohex);
		
		tratandoFlags("INC",resultadohex,valorhex1,"0000");
	}

	/**
	*Captura o valor do codigo4 e decrementa armazenando
	*o resultado no registrador representado pelo codigo2.
	*/
	public void dec()
	{
		valorhex1=capturaValor(codigo4);
		valordec1=sim.getConversao().hexa_decimal(valorhex1);
		resultadodec=valordec1-1;
		resultadohex=sim.getConversao().decimal_hexa(resultadodec);
		sim.getJanela().getReg("R"+codigo2).setValReg(resultadohex);
		
		tratandoFlags("DEC",resultadohex,valorhex1,"0000");
	}

	/**
	*Armazena no registrador representado pelo codigo2 o valor armazenado
	*na posio de memria capturada atravs do codigo4. 
	*/
	public void ld()
	{
		valorhex1=capturaValor(codigo4);
		resultadohex=sim.getMemoria().getInstrucao(valorhex1);
		sim.getJanela().getReg("R"+codigo2).setValReg(resultadohex);
		
		tratandoFlags("LD",resultadohex,valorhex1,"0000");
	}

	/**
	*Armazena no registrador representado pelo codigo2 o valor armazenado
	*na posio de memria capturada atravs do codigo3 somado com codigo4. 
	*/
	public void ld2()
	{
		valorhex1=capturaValor(codigo3);
		valorhex2=capturaValor(codigo4);
		
		valordec1=sim.getConversao().hexa_decimal(valorhex1);
		valordec2=sim.getConversao().hexa_decimal(valorhex2);
		resultadodec= valordec1+valordec2;
		resultadohex=sim.getConversao().decimal_hexa(resultadodec);

		resultadohex=sim.getMemoria().getInstrucao(resultadohex);

		sim.getJanela().getReg("R"+codigo2).setValReg(resultadohex);
		
		tratandoFlags("LD2",resultadohex,valorhex1,valorhex2);
	}

  /**
	*Armazena no registrador representado pelo codigo2 o valor armazenado
	*na posio de memria capturada atravs do SP decrementado de 1. 
	*/
	public void pop()
	{
		if(!sim.getJanela().getReg("SP").getValReg().equalsIgnoreCase(sim.getPosicaoSP()))
		{
			if(sim.getMovPilha().equalsIgnoreCase("INC"))
				sim.getJanela().getReg("SP").decrementa();
			else
				sim.getJanela().getReg("SP").incrementa();

			valorhex1=capturaValor("SP");
			resultadohex=sim.getMemoria().getInstrucao(valorhex1);	
			sim.getMemoria().setMemoria(valorhex1,"","");
			sim.getJanela().getReg("R"+codigo2).setValReg(resultadohex);
			
			tratandoFlags("POP",resultadohex,valorhex1,"0000");
		}
		else
			JOptionPane.showMessageDialog(sim.getJanela(),"POP em Pilha Vazia!","ERRO", JOptionPane.ERROR_MESSAGE);
	}

	/**
	*Armazena na posio de memria representado pelo codigo4 o valor
	*capturado do codigo3. 
	*/
	public void st()
	{
		valorhex1=capturaValor(codigo3);
		valorhex2=capturaValor(codigo4);
		sim.getVariavel().setValor(valorhex1,valorhex2,"");
		sim.getMemoria().setMemoria(valorhex1,valorhex2,"DADOS");
		
		tratandoFlags("ST",valorhex2,valorhex2,"0000");
	}

	/**
	*Armazena na posio de memria representado pelo codigo4 somado
	*ao do codigo3,o valor capturado do codigo2. 
	*/
	public void st2()
	{
		valorhex1=capturaValor(codigo3);
		valorhex2=capturaValor(codigo4);
		
		valordec1=sim.getConversao().hexa_decimal(valorhex1);
		valordec2=sim.getConversao().hexa_decimal(valorhex2);
		resultadodec= valordec1+valordec2;
		
		resultadohex=sim.getConversao().decimal_hexa(resultadodec);
		valorhex1=capturaValor(codigo2);

		sim.getVariavel().setValor(resultadohex,valorhex1,"");
		sim.getMemoria().setMemoria(resultadohex,valorhex1,"DADOS");
		
		tratandoFlags("ST2",valorhex1,resultadohex,"0000");
	}

	/**
	*Armazena na posio de memria representado por SP o valor
	*capturado do codigo3 e incrementa de 1 SP. 
	*/
	public void push()
	{
		valorhex1=capturaValor("SP");
		valorhex2=capturaValor(codigo3);
		sim.getVariavel().setValor(valorhex1,valorhex2,"");
		sim.getMemoria().setMemoria(valorhex1,valorhex2,"DADOS");
		if(sim.getMovPilha().equalsIgnoreCase("INC"))
			sim.getJanela().getReg("SP").incrementa();
		else
			sim.getJanela().getReg("SP").decrementa();
			
		tratandoFlags("ST",valorhex2,valorhex2,"0000");
	}
	
	/**
	*Seta o flag geral para TRUE se o valor capturado do registrador
	*representado pelo codigo3 for diferente do valor capturado do 
	*registrador representado pelo codigo4. Caso contrrio, seta FALSO
	*para o flag geral.
	*/
	public void dif()
	{
		String flag_dep;
		flag_dep=arq.getArqMont().procuraInstrucao("DIF").getFlag_dep();
		
		valorhex1=capturaValor(codigo3);
		valorhex2=capturaValor(codigo4);
		valordec1=sim.getConversao().hexa_decimal(valorhex1);
		valordec2=sim.getConversao().hexa_decimal(valorhex2);
		resultadodec=valordec1-valordec2;
		if(resultadodec!=0)
			sim.getJanela().getFlag(flag_dep).setValFlag(true);
		else
			sim.getJanela().getFlag(flag_dep).setValFlag(false);
		resultadohex=sim.getConversao().decimal_hexa(resultadodec);

		tratandoFlags("DIF",resultadohex,valorhex1,valorhex2);
	}

	/**
	*Seta o flag geral para TRUE se o valor capturado do registrador
	*representado pelo codigo3 for igual ao valor capturado do 
	*registrador representado pelo codigo4. Caso contrrio, seta FALSO
	*para o flag geral.
	*/
	public void eql()
	{
		String flag_dep;
		flag_dep=arq.getArqMont().procuraInstrucao("EQL").getFlag_dep();
		
		valorhex1=capturaValor(codigo3);
		valorhex2=capturaValor(codigo4);
		valordec1=sim.getConversao().hexa_decimal(valorhex1);
		valordec2=sim.getConversao().hexa_decimal(valorhex2);
		resultadodec=valordec1-valordec2;
		
		if(resultadodec==0)
			sim.getJanela().getFlag(flag_dep).setValFlag(true);
		else
			sim.getJanela().getFlag(flag_dep).setValFlag(false);

		resultadohex=sim.getConversao().decimal_hexa(resultadodec);

		tratandoFlags("EQL",resultadohex,valorhex1,valorhex2);
	}
	
	/**
	*Seta o flag geral para TRUE se o valor capturado do registrador
	*representado pelo codigo3 for superior ao valor capturado do 
	*registrador representado pelo codigo4. Caso contrrio, seta FALSO
	*para o flag geral.
	*/
	public void sup()
	{
		String flag_dep;
		flag_dep=arq.getArqMont().procuraInstrucao("SUP").getFlag_dep();
		
		valorhex1=capturaValor(codigo3);
		valorhex2=capturaValor(codigo4);
		valordec1=sim.getConversao().hexa_decimal(valorhex1);
		valordec2=sim.getConversao().hexa_decimal(valorhex2);
		resultadodec=valordec1-valordec2;
		if(resultadodec>32767)
			resultadodec=resultadodec-65536;
		
		if(resultadodec>0)
			sim.getJanela().getFlag(flag_dep).setValFlag(true);
		else
			sim.getJanela().getFlag(flag_dep).setValFlag(false);
		
		resultadohex=sim.getConversao().decimal_hexa(resultadodec);

		tratandoFlags("SUP",resultadohex,valorhex1,valorhex2);
	}
	
	/**
	*Seta o flag geral para TRUE se o valor capturado do registrador
	*representado pelo codigo3 for inferior ao valor capturado do 
	*registrador representado pelo codigo4. Caso contrrio, seta FALSO
	*para o flag geral.
	*/
	public void inf()
	{
		String flag_dep;
		flag_dep=arq.getArqMont().procuraInstrucao("INF").getFlag_dep();
		
		valorhex1=capturaValor(codigo3);
		valorhex2=capturaValor(codigo4);
		valordec1=sim.getConversao().hexa_decimal(valorhex1);
		valordec2=sim.getConversao().hexa_decimal(valorhex2);
		resultadodec=valordec1-valordec2;
		if(resultadodec>32767)
			resultadodec=resultadodec-65536;
		
		if(resultadodec<0)
			sim.getJanela().getFlag(flag_dep).setValFlag(true);
		else
			sim.getJanela().getFlag(flag_dep).setValFlag(false);

		resultadohex=sim.getConversao().decimal_hexa(resultadodec);

		tratandoFlags("INF",resultadohex,valorhex1,valorhex2);
	}
	
	/**
	*Armazena em SP o valor capturado do registrador representado pelo
	*codigo3.
	*/
	public void ldsp()
	{
		valorhex1=capturaValor(codigo3);
		sim.getJanela().getReg("SP").setValReg(valorhex1);
		
		tratandoFlags("LDSP",valorhex1,valorhex1,"0000");
	}
	/**
	*No executa nenhuma operao.
	*/
	public void nop()
	{
	}
	
	/**
	*Retorna da subrotina. Captura o valor armazenado no endereo contido
	*em SP decrementado de 1 e armazena-o em PC. 
	*/
	public void rts()
	{
		if(sim.getMovPilha().equalsIgnoreCase("INC"))
			sim.getJanela().getReg("SP").decrementa();
		else
			sim.getJanela().getReg("SP").incrementa();
		valorhex1=capturaValor("SP");
		valorhex1=sim.getMemoria().getInstrucao(valorhex1);
		sim.getJanela().getReg("PC").setValReg(valorhex1);

		tratandoFlags("RTS",valorhex1,valorhex1,"0000");
	}
	
	/**
	*Termina a execuo da simulao, desabilitando os botes STEP, RUN,
	*PAUSE e STOP.
	*/
	public void halt()
	{
		halt=true;
		sim.getJanela().getBrun().setEnabled(false);
		sim.getJanela().getBstep().setEnabled(false);
		sim.getJanela().getBstop().setEnabled(false);
		sim.getJanela().getBpause().setEnabled(false);
  	sim.getJanela().getBreset().setEnabled(true);
  	int ninst=sim.getJanela().getNInstrucoes();
  	int nclock=sim.getJanela().getNClocks();
  	int cpi=(nclock/ninst);
  	JOptionPane.showMessageDialog(sim.getJanela(),"Nmero de Instrues: "+ninst+"\nNmero de Clocks: "+nclock+"\nCPI: "+cpi,"CPI do PROGRAMA",JOptionPane.INFORMATION_MESSAGE);
	}
	
	/**
	*Se o flag geral estiver setado em TRUE realiza um jump referente
	*ao registrador representado pelo codigo3 somado ao valor do PC.
	*Esta soma  armazenada em PC.
	*/
	public void jmprc()
	{
		String flag_dep;
		flag_dep=arq.getArqMont().procuraInstrucao("JMPRC").getFlag_dep();
		
		if(sim.getJanela().getFlag(flag_dep).getValFlag())
		{
			valorhex1=capturaValor("PC");
			valorhex2=capturaValor(codigo3);
			valordec1=sim.getConversao().hexa_decimal(valorhex1);//PC
			valordec2=sim.getConversao().hexa_decimal(valorhex2);//Valor
			resultadodec=soma(valordec1,valordec2);
			resultadohex=sim.getConversao().decimal_hexa(resultadodec);
			sim.getJanela().getReg("PC").setValReg(resultadohex);
			
			tratandoFlags("JMPRC",resultadohex,valorhex1,valorhex2);
		}
	}
	
	
	/**
	*Realiza um jump incondicional referente ao registrador representado
	*pelo codigo3 somado ao valor do PC.
	*Esta soma  armazenada em PC.
	*/
	public void jmpr()
	{
			valorhex1=capturaValor("PC");
			valorhex2=capturaValor(codigo3);
			valordec1=sim.getConversao().hexa_decimal(valorhex1);//PC
			valordec2=sim.getConversao().hexa_decimal(valorhex2);//Valor
			resultadodec=soma(valordec1,valordec2);
			resultadohex=sim.getConversao().decimal_hexa(resultadodec);
			sim.getJanela().getReg("PC").setValReg(resultadohex);
			tratandoFlags("JMPR",resultadohex,valorhex1,valorhex2);
	}
	
	
	/**
	*Se o flag geral estiver setado em TRUE realiza um jump a uma
	*subrotina, armazenando o valor de PC na posicao de memria de
	*SP e armazenando em PC o seu valor somado ao valor do SP, que
	*aps  incrementado.
	*/
	public void jsrrc()
	{
		String flag_dep;
		flag_dep=arq.getArqMont().procuraInstrucao("JSRRC").getFlag_dep();
		
		if(sim.getJanela().getFlag(flag_dep).getValFlag())
		{
			valorhex1=capturaValor("PC");
			valorhex2=capturaValor("SP");

			sim.getMemoria().setMemoria(valorhex2,valorhex1,"DADOS");
			if(sim.getMovPilha().equalsIgnoreCase("INC"))
				sim.getJanela().getReg("SP").incrementa();
			else
				sim.getJanela().getReg("SP").decrementa();
				
			valorhex2=capturaValor(codigo3);
			
			valordec1=sim.getConversao().hexa_decimal(valorhex1);//PC
			valordec2=sim.getConversao().hexa_decimal(valorhex2);//Valor
			resultadodec=soma(valordec1,valordec2);
			resultadohex=sim.getConversao().decimal_hexa(resultadodec);
			sim.getJanela().getReg("PC").setValReg(resultadohex);

			tratandoFlags("JSRRC",resultadohex,valorhex1,valorhex2);
		}
	}
	
	/**
	*Realiza um jump incondicional a uma subrotina, armazenando o valor
	*de PC na posicao de memria de SP e armazenando em PC o seu valor
	*somado ao valor do SP, que aps  incrementado.
	*/
	public void jsrr()
	{
			valorhex1=capturaValor("PC");
			valorhex2=capturaValor("SP");
			sim.getMemoria().setMemoria(valorhex2,valorhex1,"DADOS");
			if(sim.getMovPilha().equalsIgnoreCase("INC"))
					sim.getJanela().getReg("SP").incrementa();
			else					
					sim.getJanela().getReg("SP").incrementa();
					
			valorhex2=capturaValor(codigo3);
			valordec1=sim.getConversao().hexa_decimal(valorhex1);//PC
			valordec2=sim.getConversao().hexa_decimal(valorhex2);//Valor
			resultadodec=soma(valordec1,valordec2);
			resultadohex=sim.getConversao().decimal_hexa(resultadodec);
			sim.getJanela().getReg("PC").setValReg(resultadohex);

			tratandoFlags("JSRR",resultadohex,valorhex1,valorhex2);
	}
	
	/**
	*Se o flag geral estiver setado em TRUE realiza um jump referente
	*ao registrador representado pelo codigo3 que  armazenado em PC.
	*/
	public void jmpc()
	{
		String flag_dep;
		flag_dep=arq.getArqMont().procuraInstrucao("JMPC").getFlag_dep();
		
		if(sim.getJanela().getFlag(flag_dep).getValFlag())
		{
			valorhex1=capturaValor(codigo3);
			sim.getJanela().getReg("PC").setValReg(valorhex1);
			
			tratandoFlags("JMPC",valorhex1,valorhex1,"0000");
		}	
	}

	/**
	*Realiza um jump incondicional referente ao registrador representado
	*pelo codigo3 que  armazenado em PC.
	*/
	public void jmp()
	{
		valorhex1=capturaValor(codigo3);
		sim.getJanela().getReg("PC").setValReg(valorhex1);

		tratandoFlags("JMP",valorhex1,valorhex1,"0000");
	}


	/**
	*Se o flag geral estiver setado em TRUE realiza um jump a uma
	*subrotina, armazenando o valor de PC na posicao de memria de
	*SP e armazenando em PC o valordo SP, que aps  incrementado.
	*/
	public void jsrc()
	{
		String flag_dep;
		flag_dep=arq.getArqMont().procuraInstrucao("JSRC").getFlag_dep();
		
		if(sim.getJanela().getFlag(flag_dep).getValFlag())
		{
			valorhex1=capturaValor("SP");
			valorhex2=capturaValor("PC");
			sim.getMemoria().setMemoria(valorhex1,valorhex2,"DADOS");
			if(sim.getMovPilha().equalsIgnoreCase("INC"))			
				sim.getJanela().getReg("SP").incrementa();
			else
				sim.getJanela().getReg("SP").decrementa();
			valorhex1=capturaValor(codigo3);
	    sim.getJanela().getReg("PC").setValReg(valorhex1);

			tratandoFlags("JSRC",valorhex1,valorhex1,valorhex2);
		}
	}

	/**
	*Realiza um jump incondicional a uma subrotina, armazenando o valor
	*de PC na posicao de memria de SP e armazenando em PC o valor do
	*SP, que aps  incrementado.
	*/
	public void jsr()
	{
			valorhex1=capturaValor("SP");
			valorhex2=capturaValor("PC");
			sim.getMemoria().setMemoria(valorhex1,valorhex2,"DADOS");
			if(sim.getMovPilha().equalsIgnoreCase("INC"))
				sim.getJanela().getReg("SP").incrementa();
			else
				sim.getJanela().getReg("SP").decrementa();
			valorhex1=capturaValor(codigo3);
			sim.getJanela().getReg("PC").setValReg(valorhex1);

			tratandoFlags("JSR",valorhex1,valorhex1,valorhex2);
	}

	/**
	*Se o flag geral estiver setado em TRUE realiza um jump referente
	*a um valor de deslocamento(12 bits) no qual  realizado a extenso
	*do sinal para posteriormente ser somado ao valor do PC e armazenado
	*no prprio PC.
	*/
	public void jmpdc()
	{
		String flag_dep;
		flag_dep=arq.getArqMont().procuraInstrucao("JMPDC").getFlag_dep();
		
		if(sim.getJanela().getFlag(flag_dep).getValFlag())
		{
			valorhex1=capturaValor("PC");
			valorhex2=ext_sinal(codigo2);
			valordec1=sim.getConversao().hexa_decimal(valorhex1); //PC
			valordec2=sim.getConversao().hexa_decimal(valorhex2); //Valor
			if(valordec2>32767)
				valordec2=-(65535-valordec2+1);
	
			resultadodec=soma(valordec1,valordec2);
			resultadohex=sim.getConversao().decimal_hexa(resultadodec);
			sim.getJanela().getReg("PC").setValReg(resultadohex);

			tratandoFlags("JMPDC",resultadohex,valorhex1,valorhex2);
		}	
	}

	/**
	*Realiza um jump incondicional referente a um valor de 
	*deslocamento(12 bits) no qual  realizado a extenso do sinal
	*para posteriormente ser somado ao valor do PC e armazenado
	*no prprio PC.
	*/
	public void jmpd()
	{
		valorhex1=capturaValor("PC");
		valorhex2=ext_sinal(codigo2);
		valordec1=sim.getConversao().hexa_decimal(valorhex1); //PC
		valordec2=sim.getConversao().hexa_decimal(valorhex2); //Valor
		resultadodec=soma(valordec1,valordec2);
		resultadohex=sim.getConversao().decimal_hexa(resultadodec);
		sim.getJanela().getReg("PC").setValReg(resultadohex);

		tratandoFlags("JMPD",resultadohex,valorhex1,valorhex2);
	}

	/**
	*Se o flag geral estiver setado em TRUE realiza um jump a uma
	*subrotina, armazenando o valor de PC na posicao de memria de
	*SP e armazenando em PC o seu valor somado ao valor do deslocamento
	*(12 bits) com a extenso do sinal.O SP posteriormente incrementado.
	*/
	public void jsrdc()
	{
		String flag_dep;
		flag_dep=arq.getArqMont().procuraInstrucao("JSRDC").getFlag_dep();
		
		if(sim.getJanela().getFlag(flag_dep).getValFlag())
		{
			valorhex1=capturaValor("PC");
			valorhex2=capturaValor("SP");
			sim.getMemoria().setMemoria(valorhex2,valorhex1,"DADOS");
			if(sim.getMovPilha().equalsIgnoreCase("INC"))
				sim.getJanela().getReg("SP").incrementa();
			else
				sim.getJanela().getReg("SP").decrementa();
	
			valorhex2=ext_sinal(codigo2);
		  valordec1=sim.getConversao().hexa_decimal(valorhex1);//PC
			valordec2=sim.getConversao().hexa_decimal(valorhex2);//Valor
			resultadodec=soma(valordec1,valordec2);
			resultadohex=sim.getConversao().decimal_hexa(resultadodec);
			sim.getJanela().getReg("PC").setValReg(resultadohex);
			
			tratandoFlags("JSRDC",resultadohex,valorhex1,valorhex2);
		}	
	}

	/**
	*Realiza um jump condicional a uma subrotina, armazenando o valor de
	*PC na posicao de memria de SP e armazenando em PC o seu valor somado
	*ao valor do deslocamento(12 bits) com a extenso do sinal.O SP
	*posteriormente incrementado.
	*/
	public void jsrd()
	{
		valorhex1=capturaValor("PC");
		valorhex2=capturaValor("SP");
		sim.getMemoria().setMemoria(valorhex2,valorhex1,"DADOS");

		if(sim.getMovPilha().equalsIgnoreCase("INC"))
			sim.getJanela().getReg("SP").incrementa();
		else
			sim.getJanela().getReg("SP").decrementa();
	
		valorhex2=ext_sinal(codigo2);
	  valordec1=sim.getConversao().hexa_decimal(valorhex1);//PC
		valordec2=sim.getConversao().hexa_decimal(valorhex2);//Valor
		resultadodec=soma(valordec1,valordec2);
		resultadohex=sim.getConversao().decimal_hexa(resultadodec);
		sim.getJanela().getReg("PC").setValReg(resultadohex);
		
		tratandoFlags("JSRDC",resultadohex,valorhex1,valorhex2);
	}

	/**
	*Se o flag negativo estiver setado em TRUE realiza um jump referente
	*ao registrador representado pelo codigo4 que  armazenado em PC.
	*/
	public void jn()
	{
		if(sim.getJanela().getFlag("NEGATIVO").getValFlag())
		{
			valorhex1=capturaValor(codigo4);
			sim.getJanela().getReg("PC").setValReg(valorhex1);
			
			tratandoFlags("JN",valorhex1,valorhex1,"0000");
		}			
	}

	/**
	*Se o flag de zero estiver setado em TRUE realiza um jump referente
	*ao registrador representado pelo codigo4 que  armazenado em PC.
	*/
	public void jz()
	{
		if(sim.getJanela().getFlag("ZERO").getValFlag())
		{
			valorhex1=capturaValor(codigo4);
			sim.getJanela().getReg("PC").setValReg(valorhex1);
			
			tratandoFlags("JZ",valorhex1,valorhex1,"0000");			
		}			
	}
	
	/**
	*Se o flag de carry estiver setado em TRUE realiza um jump referente
	*ao registrador representado pelo codigo4 que  armazenado em PC.
	*/
	public void jc()
	{
		if(sim.getJanela().getFlag("CARRY").getValFlag())
		{
			valorhex1=capturaValor(codigo4);
			sim.getJanela().getReg("PC").setValReg(valorhex1);
			
			tratandoFlags("JC",valorhex1,valorhex1,"0000");
		}			
	}
	
	/**
	*Se o flag de overflow estiver setado em TRUE realiza um jump referente
	*ao registrador representado pelo codigo4 que  armazenado em PC.
	*/
	public void jv()
	{
		if(sim.getJanela().getFlag("OVERFLOW").getValFlag())
		{
			valorhex1=capturaValor(codigo4);
			sim.getJanela().getReg("PC").setValReg(valorhex1);

			tratandoFlags("JV",valorhex1,valorhex1,"0000");			
		}			
	}

	/**
	*Se o flag negativo estiver setado em TRUE realiza um jump referente
	*ao registrador representado pelo codigo4 somado com PC e armazenado
	*em PC.
	*/
	public void jnr()
	{
		if(sim.getJanela().getFlag("NEGATIVO").getValFlag())
		{
			valorhex1=capturaValor(codigo4);
			valorhex2=capturaValor("PC");
			valordec1=sim.getConversao().hexa_decimal(valorhex1);
			valordec2=sim.getConversao().hexa_decimal(valorhex2);
			resultadodec=soma(valordec1,valordec2);
			resultadohex=sim.getConversao().decimal_hexa(resultadodec);
			sim.getJanela().getReg("PC").setValReg(valorhex1);
			
			tratandoFlags("JNR",valorhex1,valorhex1,valorhex2);
		}			
	}

	/**
	*Se o flag de zero estiver setado em TRUE realiza um jump referente
	*ao registrador representado pelo codigo4 somado com PC e armazenado
	*em PC.
	*/
	public void jzr()
	{
		if(sim.getJanela().getFlag("ZERO").getValFlag())
		{
			valorhex1=capturaValor(codigo4);
			valorhex2=capturaValor("PC");
			valordec1=sim.getConversao().hexa_decimal(valorhex1);
			valordec2=sim.getConversao().hexa_decimal(valorhex2);
			resultadodec=soma(valordec1,valordec2);
			resultadohex=sim.getConversao().decimal_hexa(resultadodec);
			sim.getJanela().getReg("PC").setValReg(valorhex1);
						
			tratandoFlags("JZR",valorhex1,valorhex1,valorhex2);
		}			
	}
	
	/**
	*Se o flag de carry estiver setado em TRUE realiza um jump referente
	*ao registrador representado pelo codigo4 somado com PC e armazenado
	*em PC.
	*/
	public void jcr()
	{
		if(sim.getJanela().getFlag("CARRY").getValFlag())
		{
			valorhex1=capturaValor(codigo4);
			valorhex2=capturaValor("PC");
			valordec1=sim.getConversao().hexa_decimal(valorhex1);
			valordec2=sim.getConversao().hexa_decimal(valorhex2);
			resultadodec=soma(valordec1,valordec2);
			resultadohex=sim.getConversao().decimal_hexa(resultadodec);
			sim.getJanela().getReg("PC").setValReg(valorhex1);
						
			tratandoFlags("JCR",valorhex1,valorhex1,valorhex2);
		}			
	}
	
	/**
	*Se o flag de overflow estiver setado em TRUE realiza um jump referente
	*ao registrador representado pelo codigo4 somado com PC e armazenado
	*em PC.
	*/
	public void jvr()
	{
		if(sim.getJanela().getFlag("OVERFLOW").getValFlag())
		{
			valorhex1=capturaValor(codigo4);
			valorhex2=capturaValor("PC");
			valordec1=sim.getConversao().hexa_decimal(valorhex1);
			valordec2=sim.getConversao().hexa_decimal(valorhex2);
			resultadodec=soma(valordec1,valordec2);
			resultadohex=sim.getConversao().decimal_hexa(resultadodec);
			sim.getJanela().getReg("PC").setValReg(valorhex1);
									
			tratandoFlags("JVR",valorhex1,valorhex1,valorhex2);
		}			
	}

	/**
	*Se o flag negativo estiver setado em TRUE realiza um jump referente
	*ao deslocamento com extenso de sinal que  somado com PC e armazenado
	*em PC.
	*/
	public void jnd()
	{
		if(sim.getJanela().getFlag("NEGATIVO").getValFlag())
		{
			valorhex1=ext_sinal(codigo2);
		  valorhex2=capturaValor("PC");
			valordec1=sim.getConversao().hexa_decimal(valorhex1);
			valordec2=sim.getConversao().hexa_decimal(valorhex2);
			resultadodec=soma(valordec1,valordec2);
			resultadohex=sim.getConversao().decimal_hexa(resultadodec);
			sim.getJanela().getReg("PC").setValReg(valorhex1);
						
			tratandoFlags("JND",valorhex1,valorhex1,valorhex2);
		}			
	}

	/**
	*Se o flag de zero estiver setado em TRUE realiza um jump referente
	*ao deslocamento com extenso de sinal que  somado com PC e armazenado
	*em PC.
	*/
	public void jzd()
	{
		if(sim.getJanela().getFlag("ZERO").getValFlag())
		{
			valorhex1=ext_sinal(codigo2);
		  valorhex2=capturaValor("PC");
			valordec1=sim.getConversao().hexa_decimal(valorhex1);
			valordec2=sim.getConversao().hexa_decimal(valorhex2);
			resultadodec=soma(valordec1,valordec2);
			resultadohex=sim.getConversao().decimal_hexa(resultadodec);
			sim.getJanela().getReg("PC").setValReg(valorhex1);

			tratandoFlags("JZD",valorhex1,valorhex1,valorhex2);
		}			
	}
	
	/**
	*Se o flag de carry estiver setado em TRUE realiza um jump referente
	*ao deslocamento com extenso de sinal que  somado com PC e armazenado
	*em PC.
	*/
	public void jcd()
	{
		if(sim.getJanela().getFlag("CARRY").getValFlag())
		{
			valorhex1=ext_sinal(codigo2);
		  valorhex2=capturaValor("PC");
			valordec1=sim.getConversao().hexa_decimal(valorhex1);
			valordec2=sim.getConversao().hexa_decimal(valorhex2);
			resultadodec=soma(valordec1,valordec2);
			resultadohex=sim.getConversao().decimal_hexa(resultadodec);
			sim.getJanela().getReg("PC").setValReg(valorhex1);

			tratandoFlags("JCD",valorhex1,valorhex1,valorhex2);			
		}			
	}
	
	/**
	*Se o flag de overflow estiver setado em TRUE realiza um jump referente
	*ao deslocamento com extenso de sinal que  somado com PC e armazenado
	*em PC.
	*/
	public void jvd()
	{
		if(sim.getJanela().getFlag("OVERFLOW").getValFlag())
		{
			valorhex1=ext_sinal(codigo2);
		  valorhex2=capturaValor("PC");
			valordec1=sim.getConversao().hexa_decimal(valorhex1);
			valordec2=sim.getConversao().hexa_decimal(valorhex2);
			resultadodec=soma(valordec1,valordec2);
			resultadohex=sim.getConversao().decimal_hexa(resultadodec);
			sim.getJanela().getReg("PC").setValReg(valorhex1);

			tratandoFlags("JVD",valorhex1,valorhex1,valorhex2);			
		}			
	}
	
	/**
	*Trata os flags da instruo.
	*/
	public void tratandoFlags(String inst,String resultado,String valorhex1,String valorhex2)
	{
		String [] flags;
		NodoArqMont aux=arq.getArqMont().procuraInstrucao(inst);
		flags=aux.getFlags();
		
		for(int i=0;i<aux.getNflag();i++)
		{
			if(flags[i]!=null && flags[i].equalsIgnoreCase("GERAL"))
				flagGeral(inst,resultado);
			if(flags[i]!=null && flags[i].equalsIgnoreCase("ZERO"))
				flagZero(resultado);
			if(flags[i]!=null && flags[i].equalsIgnoreCase("CARRY"))
				flagCarry(inst,valorhex1,valorhex2);
			if(flags[i]!=null && flags[i].equalsIgnoreCase("NEGATIVO"))
				flagNegativo(resultado);
			if(flags[i]!=null && flags[i].equalsIgnoreCase("OVERFLOW"))
        flagOverflow(inst,valorhex1,valorhex2);
		}
	}

	public void flagGeral(String inst,String resultado)
	{
	}
	
	/**
	* Seta o flag de Zero para um, quando o valor  zero e para zero
	* quando o valor  diferente de zero. 
	*/
	public void flagZero(String resultado)
	{
		int resultadodec;

		resultadodec=sim.getConversao().hexa_decimal(resultado);
		if(resultadodec==0)
			sim.getJanela().getFlag("ZERO").setValFlag(true);
		else
			sim.getJanela().getFlag("ZERO").setValFlag(false);
	}
	
	
	/**
	* Seta o flag de Carry para um, quando a operacao no bit mais 
	* significativo resultou em um carry, caso contrario seta o flag
	* para zero.
	*/
	public void flagCarry(String inst,String valorhex1,String valorhex2)
	{
		int valordec1,valordec2;
		String valorbin1,valorbin2;
		
		valordec1=sim.getConversao().hexa_decimal(valorhex1);
		valordec2=sim.getConversao().hexa_decimal(valorhex2);
		
		valorbin1=sim.getConversao().decimal_binario(valordec1);
		valorbin2=sim.getConversao().decimal_binario(valordec2);

		if(inst.equalsIgnoreCase("ADD"))
		{
			if(valorbin1.substring(0,1).equals("1") && valorbin2.substring(0,1).equals("1"))
			  sim.getJanela().getFlag("CARRY").setValFlag(true);
			else
				sim.getJanela().getFlag("CARRY").setValFlag(false);
		}
	}
	
	/**
	* Seta o Flag de Negativo para um quando o valor for menor que
	* zero, caso contrrio, seta o flag para zero.
	*/
	public void flagNegativo(String resultado)
	{
		int resultadodec;

		resultadodec=sim.getConversao().hexa_decimal(resultado);

		if(resultadodec>32767)
			sim.getJanela().getFlag("NEGATIVO").setValFlag(true);
		else
			sim.getJanela().getFlag("NEGATIVO").setValFlag(false);		
	}
	
	/**
	* Seta o Flag de Overflow para um quando o valor for menor que
	* zero, caso contrrio, seta o flag para zero.
	*/
	public void flagOverflow(String inst,String valorhex1,String valorhex2)
	{
		int resultadodec,valordec1,valordec2;
		
		valordec1=sim.getConversao().hexa_decimal(valorhex1);
    valordec2=sim.getConversao().hexa_decimal(valorhex2);

		resultadodec=valordec1+valordec2;

		if(resultadodec>65535)
			sim.getJanela().getFlag("OVERFLOW").setValFlag(true);
		else
			sim.getJanela().getFlag("OVERFLOW").setValFlag(false);
	}
	
	/**
	*Concatena ZEROs a esquerda se o bit mais significativo  ZERO, e
	*concatena UMs caso o bit mais significativo seja UM.
	*/
	public String ext_sinal(String codigo)
	{
		String valor="";
				
		valordec1=sim.getConversao().hexa_decimal(codigo);
		
		if(valordec1<8)
		{
			if(codigo.equals(codigo2))
				valor=("0");
			else if(codigo.equals(codigo3))
				valor=("00");
			else if(codigo.equals(codigo3))
				valor=("000");
		}
		else	
		{
			if(codigo.equals(codigo2))
				valor=("F");
			else if(codigo.equals(codigo3))
				valor=("FF");
			else if(codigo.equals(codigo3))
				valor=("FFF");
		}
		
		if(codigo.equals(codigo2))
			valor=valor.concat(codigo2);
		if(codigo.equals(codigo2)||codigo.equals(codigo3))
			valor=valor.concat(codigo3);
		if(codigo.equals(codigo2)||codigo.equals(codigo3)||codigo.equals(codigo4))
			valor=valor.concat(codigo4);

		return valor;
	}
	
	/**
	*Captura o valor armazenado no registrador referente ao codigo. 
	*/
	public String capturaValor(String codigo)
	{
		String valor;

		if(codigo.equals("PC"))
			valor=sim.getJanela().getReg("PC").getValReg();
		else if(codigo.equals("SP"))
			valor=sim.getJanela().getReg("SP").getValReg();
		else		
			valor=sim.getJanela().getReg("R"+codigo).getValReg();
	
		if(valor.equals(""))
			valor="0000";
		return valor;
	}	
	
	/**
	*Executa a operao lgica AND.
	*/
	public String e(String valorbin1,String valorbin2)
	{
		String resultado,res;
		String val1,val2;
		
		resultado="";
		
		for(int bit=16;bit>0;bit--)
		{
			val1=valorbin1.substring(bit-1,bit);
			val2=valorbin2.substring(bit-1,bit);
			if(val1.equals("0")||val2.equals("0"))
				res="0";
			else
				res="1";
			resultado=res.concat(resultado);
		}
		return resultado;
	}
	
	/**
	*Executa a operao lgica OR.
	*/
	public String or(String valorbin1,String valorbin2)
	{
		String resultado,res;
		String val1,val2;
		
		resultado="";
		
		for(int bit=16;bit>0;bit--)
		{
			val1=valorbin1.substring(bit-1,bit);
			val2=valorbin2.substring(bit-1,bit);
			if(val1.equals("1")||val2.equals("1"))
				res="1";
			else
				res="0";
			resultado=res.concat(resultado);
		}
		return resultado;
	}

	/**
	*Executa a operao lgica XOR.
	*/
	public String xor(String valorbin1,String valorbin2)
	{
		String resultado,res;
		String val1,val2;
		
		resultado="";
		
		for(int bit=16;bit>0;bit--)
		{
			val1=valorbin1.substring(bit-1,bit);
			val2=valorbin2.substring(bit-1,bit);
			if((val1.equals("0")&&val2.equals("0"))||(val1.equals("1")&&val2.equals("1")))
				res="0";
			else
				res="1";
			resultado=res.concat(resultado);
		}
		return resultado;
	}
	
	/**
	*Soma os valores e se o resultado for superior ao tamanho da memria,
	*o resultado  subtrado do tamanho da memria diminuido de 1.
	*/
	public int soma(int valor1,int valor2)
	{	
		int resultado;
		
		resultado=valor1+valor2;
    

		return resultado;
	}		
}