/*
* @(#)Janela.java  1.0  30/03/2001
*	
* Aline Vieira de Mello
* Pontifcia Universidade Catlica do Rio Grande do Sul - Faculdade de Informtica
*/

import javax.swing.*;
import javax.swing.table.*;
import java.awt.event.*;
import java.awt.*;

/**
*<i>Janela</i> Implementa a janela para a simulao com os menus Arquivo,
*Parmetros e Ajuda. Implementa tambm, as tabelas de memria e de
*variveis, os dezesseis registradores mais o PC e o SP, o flag geral e
*os botes de controle da simulao.
*/
public class Janela extends JFrame implements ActionListener
{
	private Simulador sim;
	private Tabela tabelamemoria,tabelavariavel;
	private JFrame fMensagem;
	private JLabel velocidade,lMensagem;
	private JPanel panelcenter;
	private JRadioButton lento,normal,rapido;
	private JButton bstep,brun,bstop,bpause,breset;
	private JScrollPane scrollPaneMemoria,scrollPaneVariaveis;
	private ButtonGroup grupo;
	private MeuMenu menu;
	private Registrador reg[];
	private Flag flag[];
	private Object[][] dadosVariaveis,dadosMemoria;
	private String[] nomeColunasVariaveis,nomeColunasMemoria,flags;
	private int[] dimColunasMemoria,dimColunasVariaveis;
	private int nreg;
	private int nflag;
	private double tempo;
	private boolean step;
	private boolean run;
	private boolean pause;
	private boolean stop;
	private boolean reset;
	private int nclock,ninstrucao;
	  	    
	public Janela(Simulador sim,int nreg,int nflag,String[] flags) 
  {
  	super();
 		this.sim=sim;
 		this.nreg=nreg;
 		this.nflag=nflag;
 		this.flags=flags;
 		nclock=0;
 		ninstrucao=0;
 		setTitle(sim.getNomeArquitetura());
 		getContentPane().setLayout(new BorderLayout());
	  setSize(770,540);
	  setLocation(20,15);
    addWindowListener(new WindowAdapter(){public void windowClosing(WindowEvent e){System.exit(0);}});
    menu=new MeuMenu(this);
  	reg=new Registrador[nreg+3];
  	flag=new Flag[nflag];
		adicionaPanelCenter();
		setVisible(true);
	}

	/**
	*Captura o valor booleano do STEP.
	*/
	public boolean getStep(){return step;} 
	
	/**
	*Captura o valor booleano do RUN.
	*/
	public boolean getRun(){return run;}
	
	/**
	*Captura o valor booleano do PAUSE.
	*/
	public boolean getPause(){return pause;}	
	
	/**
	*Captura o valor booleano do STOP.
	*/
	public boolean getStop(){return stop;}
	
	/**
	*Captura o valor booleano do RESET.
	*/
	public boolean getReset(){return reset;}
			   
	/**
	*Captura o boto Registrador corresponde ao nome informado.
	*/
	public Registrador getReg(String nome)
	{
		if(nome.equalsIgnoreCase("RA"))
			nome="R10";
		else if(nome.equalsIgnoreCase("RB"))
			nome="R11";
		else if(nome.equalsIgnoreCase("RC"))
			nome="R12";
		else if(nome.equalsIgnoreCase("RD"))
			nome="R13";
		else if(nome.equalsIgnoreCase("RE"))
			nome="R14";
		else if(nome.equalsIgnoreCase("RF"))
			nome="R15";
						
		//Registrador R0 a R15
		for(int indice=0;indice<nreg;indice++)
		{
			if(reg[indice].getNome().equalsIgnoreCase(nome))
				return reg[indice];
		}	
		
		//Registrador IR
		if(reg[nreg].getNome().equalsIgnoreCase(nome))
			return reg[nreg];
		
		//Registrador PC
		if(reg[nreg+1].getNome().equalsIgnoreCase(nome))
			return reg[nreg+1];
		
		//Registrador SP
		if(reg[nreg+2].getNome().equalsIgnoreCase(nome))
			return reg[nreg+2];

		return null;
	}
	
	/**
	*Retorna o registrador indicado pelo indice informado. 
	*/
	public Registrador getReg(int i){return reg[i];}
	
	/**
	*Captura o tempo para simulao.
	*/
	public double getTempo()
	{
		if(lento.isSelected())
			tempo=3.0;
		else if(normal.isSelected())
			tempo=1.0;
		else if(rapido.isSelected())
			tempo=0;
		return tempo;
	}
	
	/**
	*Retorna o flag indicado pelo indice informado. 
	*/
	public Flag getFlag(int i){return flag[i];}
	
	/**
	*Captura o flag correspondente ao nome indicado.
	*/
	public Flag getFlag(String n)
	{
		for(int i=0;i<nflag;i++)
		{
			if(flag[i].getNome().equalsIgnoreCase(n))
				return flag[i];
		}
		return null;
	}
	
	/**
	*Captura a tabela Memria.
	*/
	public JTable getTabelaMemoria() {return tabelamemoria; }
	
	/**
	*Captura a tabela de Variveis.
	*/
	public JTable getTabelaVariavel() {return tabelavariavel; }
	
	/**
	*Captura o boto STEP.
	*/	
	public JButton getBstep(){return bstep;}
	
	/**
	*Captura o boto RESET.
	*/	
	public JButton getBreset(){return breset;}
	
	/**
	*Captura o boto STOP.
	*/	
	public JButton getBstop(){return bstop;}
	
	/**
	*Captura o boto RUN.
	*/	
	public JButton getBrun(){return brun;}
	
	/**
	*Captura o boto PAUSE.
	*/	
	public JButton getBpause(){return bpause;}
	
	/**
	*Captura o panel central.
	*/	
	public JPanel getPanel(){return panelcenter;}
	
	/**
	*Captura o tamanho da memria.
	*/	
	public int getTamanhoMemoria(){return sim.getTamanhoMemoria();}
	
	/**
	*Captura o nmero mximo de variveis definido.
	*/	
	public int getNumeroVariaveis(){return sim.getNumeroVariaveis();}
	
	/**
	*Captura o nmero de instrues do programa.
	*/	
	public int getNInstrucoes(){return ninstrucao;}
	
	/**
	*Captura o nmero de clocks do programa.
	*/	
	public int getNClocks(){return nclock;}
	
	public JScrollPane getScrollPaneMemoria(){return scrollPaneMemoria;}
	
	public void incrementaNclock(int n){nclock=nclock+n;}
	public void incrementaNinstrucao(){ninstrucao++;}

	/**
	*Adiciona o panel central com os dezesseis registradores, o pc, o
	*sp, o flag geral, as tabelas de memria e variveis e os botes
	*de controle.
	*/
	public void adicionaPanelCenter()
	{
		panelcenter =new JPanel();
		panelcenter.setLayout(null);
		
		implementaRegistradores();
		
		implementaFlag();
		
		inicializaRegistradoresFlag();
		
		implementaTempo();
		
		implementaDadosTabelas();
		
		implementaTabelaMemoria();
		
		implementaTabelaVariaveis();
		
		implementaBotoes();
		
		getContentPane().add(panelcenter,BorderLayout.CENTER);
	}

	/**
	*Implementa os dezesseis registradores de uso geral, mais o pc e o
	*sp.
	*/
	public void implementaRegistradores()
	{
		int tam=5;
		int x=580;
		int y=30;
		int dimx=40;
		int dimy=20;
		int somay,somay1,somay2;
		int indice;
	
		//Registradores de R0 a R7
		for(indice=0,somay1=0;indice<((nreg+1)/2);indice++,somay1=somay1+30)
			reg[indice]=new Registrador(sim,panelcenter,"R"+indice,x,y+somay1,dimx,dimy,"Registrador R"+indice);
		
		//Registradores de R8 a R15
		for(indice=((nreg+1)/2),somay2=0;indice<(nreg);indice++,somay2=somay2+30)
			reg[indice]=new Registrador(sim,panelcenter,"R"+indice,x+80,y+somay2,dimx,dimy,"Registrador R"+indice);
			
		if(somay1>somay2)
			somay=somay1+10;
		else
			somay=somay1+10;
		
		//IR
			reg[nreg]=new Registrador(sim,panelcenter,"IR",x,y+somay,dimx,dimy,"Registrador de Instruo");

		//PC
			reg[nreg+1]=new Registrador(sim,panelcenter,"PC",x+80,y+somay,dimx,dimy,"Program Counter");
		
		somay=somay+30;
		//SP
			reg[nreg+2]=new Registrador(sim,panelcenter,"SP",x,y+somay,dimx,dimy,"Stack Pointer");
		
		JLayeredPane lbRegistradores=new JLayeredPane();
		lbRegistradores.setPreferredSize(new Dimension(175,somay+50));
		lbRegistradores.setBounds(x-15,y-20,175,somay+50);
		lbRegistradores.setBorder(BorderFactory.createTitledBorder("Registradores"));
		panelcenter.add(lbRegistradores);
	}
	
	/**
	*Implementa o flag geral.
	*/
	public void implementaFlag()
	{
		int x=580;
		int y=390;
		int dimx=70;
		int dimy=20;
		int somay1,somay2,indice;
		
		for(indice=0,somay1=0;indice<((nflag+1)/2);indice++,somay1=somay1+20)
			flag[indice]=new Flag(panelcenter,flags[indice],x,y+somay1,dimx,dimy,"Flag de "+flags[indice]);
		
		for(indice=((nflag+1)/2),somay2=0;indice<nflag;indice++,somay2=somay2+20)
				flag[indice]=new Flag(panelcenter,flags[indice],x+80,y+somay2,dimx,dimy,"Flag de "+flags[indice]);
		
		JLayeredPane lbFlag=new JLayeredPane();
		if(somay1>somay2)
		{
			lbFlag.setPreferredSize(new Dimension(175,somay1+30));
			lbFlag.setBounds(x-15,y-20,175,somay1+30);
		}				
		else
		{
			lbFlag.setPreferredSize(new Dimension(175,somay2+30));
			lbFlag.setBounds(x-15,y-20,175,somay2+30);
		}
		lbFlag.setBorder(BorderFactory.createTitledBorder("Flags"));
		panelcenter.add(lbFlag);
	}
	
	/**
	*Implementa os botes que controlam a velocidade da simulao.
	*/
	public void implementaTempo()
	{
		int x=90;
		int y=440;
		velocidade=new JLabel("Velocidade:");
		velocidade.setBounds(x,y,80,20);
		panelcenter.add(velocidade);
		lento=new JRadioButton("Lento",false);
		lento.setBounds(x+85,y,100,20);
		lento.setToolTipText("Velocidade lenta do simulador.");
		panelcenter.add(lento);
  	normal=new JRadioButton("Normal",true);
		normal.setBounds(x+190,y,100,20);
		normal.setToolTipText("Velocidade normal do simulador.");
		panelcenter.add(normal);
		rapido=new JRadioButton("Rpido",false);
		rapido.setBounds(x+295,y,100,20);
		rapido.setToolTipText("Velocidade rpida do simulador.");
		panelcenter.add(rapido);
		
		grupo=new ButtonGroup();
		grupo.add(lento);
		grupo.add(normal);
		grupo.add(rapido);
	}
	
	/**
	*Instncia os dados iniciais das tabelas Memria e Variveis.
	*/
	public void implementaDadosTabelas()
	{
		//DADOS DA MEMORIA
		dadosMemoria=new Object[sim.getTamanhoMemoria()][3];
		for (int i=0;i<sim.getTamanhoMemoria();i++)
		{
			dadosMemoria[i][0]="";
			dadosMemoria[i][1]=sim.getConversao().decimal_hexa(i);
			dadosMemoria[i][2]="";
		}
	
		//DADOS DAS VARIAVEIS
		dadosVariaveis=new Object[sim.getNumeroVariaveis()][3];
		for (int i=0;i<sim.getNumeroVariaveis();i++)
		{
			dadosVariaveis[i][0]="";
			dadosVariaveis[i][1]="";
			dadosVariaveis[i][2]="";
		}
	}
	
	/**
	*Implementa as tabelas Memria e Variveis.
	*/
	public void implementaTabelaVariaveis()
	{
		int x=335;
		int y=30;
		
	  nomeColunasVariaveis = new String[3];
    nomeColunasVariaveis[0]="Simbolo";
    nomeColunasVariaveis[1]="Endereco";
    nomeColunasVariaveis[2]="Valor";
	  
	  dimColunasVariaveis=new int[3];
	  dimColunasVariaveis[0]=100;
  	dimColunasVariaveis[1]=50;
    dimColunasVariaveis[2]=50;

		JLayeredPane lbVariaveis=new JLayeredPane();
		lbVariaveis.setPreferredSize(new Dimension(230,360));
		lbVariaveis.setBounds(x-15,y-20,230,360);
		lbVariaveis.setBorder(BorderFactory.createTitledBorder("Smbolos"));
		panelcenter.add(lbVariaveis);
	
		tabelavariavel = new Tabela(dadosVariaveis,nomeColunasVariaveis,sim.getNumeroVariaveis(),3,dimColunasVariaveis);
		scrollPaneVariaveis = new JScrollPane(tabelavariavel);
		scrollPaneVariaveis.setBounds(x,y,200,330);
		panelcenter.add(scrollPaneVariaveis);
	}
	
	/**
	*Implementa a tebela que contem a memria.
	*/
	public void implementaTabelaMemoria()
	{
  	int x=25;
  	int y=30;

		nomeColunasMemoria=new String[3];
		nomeColunasMemoria[0]="Instruo";
		nomeColunasMemoria[1]="Endereo";
		nomeColunasMemoria[2]="Cdigo";

		dimColunasMemoria=new int[3];
    dimColunasMemoria[0]=130;
    dimColunasMemoria[1]=55;
    dimColunasMemoria[2]=55;
		
		JLayeredPane lbMemoria=new JLayeredPane();
		lbMemoria.setPreferredSize(new Dimension(295,360));
		lbMemoria.setBounds(x-15,y-20,295,360);
		lbMemoria.setBorder(BorderFactory.createTitledBorder("Memria"));
		panelcenter.add(lbMemoria);
		
		tabelamemoria = new Tabela(dadosMemoria,nomeColunasMemoria,sim.getTamanhoMemoria(),3,dimColunasMemoria);
	  scrollPaneMemoria = new JScrollPane(tabelamemoria);
		scrollPaneMemoria.setBounds(x,y,265,330);
		panelcenter.add(scrollPaneMemoria);
	}
	
	/**
	*inicializa as tabelas de memoria e variavel.
	*/
	public void inicializaTabelas()
	{
		implementaDadosTabelas();
		tabelamemoria.inicializa(dadosMemoria);
		tabelavariavel.inicializa(dadosVariaveis);
	}
	
	/**
	*Inicializa os registradores e flags.
	*/
	public void inicializaRegistradoresFlag()
	{	
	  for(int i=0;i<nreg;i++)
		  getReg("R"+i).setValReg("0000");

		getReg("IR").setValReg("0000");
		getReg("PC").setValReg("0000");
		getReg("SP").setValReg(sim.getPosicaoSP());
		for(int i=0;i<nflag;i++)
			getFlag(i).setValFlag(false);
	}
	
	/**
	*Inicializa as tabelas, os flags e os registradores.
	*/
	public void inicializa()
	{
		String title=getTitle();
    setTitle(sim.getNomeArquitetura()+"                      REINICIALIZANDO OS DADOS PARA SIMULAO");
    inicializaTabelas();
		inicializaRegistradoresFlag();
		ninstrucao=0;
		nclock=0;
		setTitle(title);
	}		
	
		
	/**
	*Inicializa a simulao.
	*/
	public void dimensionaTamanhoMemoria()
	{
		menu=new MeuMenu(this);
		panelcenter.remove(scrollPaneMemoria);
		implementaTabelaMemoria();
		
		inicializaRegistradoresFlag();
	}

  /**
	*Inicializa a simulao.
	*/
	public void dimensionaNumeroVariaveis()
	{
		menu=new MeuMenu(this);
		panelcenter.remove(scrollPaneVariaveis);
		implementaTabelaVariaveis();
	
  	inicializaRegistradoresFlag();
	}
	
	/**
	*Implementa os botes que controlam a simulao.
	*/
	public void implementaBotoes()
	{
		int x=60;
		bstep=new JButton("STEP");
		bstep.setBounds(x,400,80,30);
		bstep.setToolTipText("Executa passo a passo.");
		bstep.addActionListener(this);
		bstep.setEnabled(false);
		panelcenter.add(bstep);
		brun=new JButton("RUN");
		brun.setBounds(x+85,400,80,30);
		brun.setToolTipText("Executa.");
		brun.addActionListener(this);
		brun.setEnabled(false);
		panelcenter.add(brun);
		bpause=new JButton("PAUSE");
		bpause.setBounds(x+170,400,80,30);
		bpause.setToolTipText("Para a execuo.");
		bpause.addActionListener(this);
		bpause.setEnabled(false);
		panelcenter.add(bpause);
		bstop=new JButton("STOP");
		bstop.setBounds(x+255,400,80,30);
		bstop.setToolTipText("Termina a execuo.");
		bstop.addActionListener(this);
		bstop.setEnabled(false);
		panelcenter.add(bstop);
		breset=new JButton("RESET");
		breset.setBounds(x+340,400,80,30);
		breset.setToolTipText("Reinicializa para nova execuo.");
		breset.addActionListener(this);
		breset.setEnabled(false);
  	panelcenter.add(breset);
	}
	
	/**
	*Executa a simulao da instruo atual do pc.
	*/
	public void step()
	{
		step=true;
		run=false;
		stop=false;
		pause=false;
		reset=false;
		Execucao exec=new Execucao(sim,sim.getArquitetura());
		exec.start();
	}
	
	/**
	*Executa a simulao das instrues.
	*/
	public void run()
	{
		step=false;
		run=true;
		stop=false;
		pause=false;
		reset=false;
		brun.setEnabled(false);
		bstep.setEnabled(false);
		breset.setEnabled(false);
		bpause.setEnabled(true);
		Execucao exec=new Execucao(sim,sim.getArquitetura());
		exec.start();
	}
	
	/**
	*Produz uma pausa na execuo da simulao.
	*/
	public void pause()
	{
		step=false;
		run=false;
		stop=false;
		pause=true;
		reset=false;
		brun.setEnabled(true);
		bstep.setEnabled(true);
		bpause.setEnabled(false);
	}
	
	/**
	*Termina a execuo da simulao.
	*/
	public void stop()
	{
		step=false;
		run=false;
		stop=true;
		pause=false;
		reset=false;
		brun.setEnabled(true);
		bstep.setEnabled(true);
		breset.setEnabled(true);
  	bpause.setEnabled(false);
	}
	
	/**
	*Reinicializa a simulao.
	*/
	public void reset()
	{
    step=false;
		run=false;
		stop=false;
		pause=false;
		reset=true;
		bpause.setEnabled(false);
 	  inicializa();
	  sim.getExecucao().setHalt(false);
		tabelamemoria.setRowSelectionInterval(0,0);
		scrollPaneMemoria.getVerticalScrollBar().setValue(0);
		sim.getArquivo().carregar();
	}	
	
	/**
	*Executa uma operao conforme o boto de controle que for pressionado.
	*/
	public void actionPerformed(ActionEvent e)
  {
	   if(e.getActionCommand().equalsIgnoreCase("STEP"))
	   		step();
	   else if(e.getActionCommand().equalsIgnoreCase("RUN"))
	   		run();
	   else if(e.getActionCommand().equalsIgnoreCase("PAUSE"))
	   		pause();
	   else if(e.getActionCommand().equalsIgnoreCase("STOP"))
	   		stop();
	   else if(e.getActionCommand().equalsIgnoreCase("RESET"))
	   		reset();
	   else	if(e.getActionCommand().equalsIgnoreCase("Abrir"))
    		sim.getArquivo().abrir();
     else if(e.getActionCommand().equalsIgnoreCase("Sair"))
    		System.exit(0);
		 else if(e.getActionCommand().equalsIgnoreCase("Sobre"))
     {
     		JOptionPane.showMessageDialog(sim.getJanela(),"Simulador de Arquiteturas       06.07.2001\nAline Vieira de Mello","VERSO 0.3",JOptionPane.INFORMATION_MESSAGE);
		 }
		 else if(e.getActionCommand().equalsIgnoreCase("Ajuda"))
     {
		    Help help=new Help(sim);
    		help.apresenta();
		 }
  }
}